using System;
using System.Runtime.InteropServices;

namespace Lydos.Matlab
{
    /// <summary>
    /// Bindings for the unmanaged Matlab API.
    /// </summary>
    internal class NativeMethods
    {
        #region Private constructor
        private NativeMethods() { }
        #endregion

        #region Libeng DLL imports
        [DllImport("libeng.dll")]
        public static extern int engClose(IntPtr ep);

        [DllImport("libeng.dll")]
        public static extern int engEvalString(IntPtr ep, string str);

        [DllImport("libeng.dll")]
        public static extern IntPtr engGetVariable(IntPtr ep, string name);

        [DllImport("libeng.dll")]
        public static extern int engGetVisible(IntPtr ep, [MarshalAs(UnmanagedType.U1)] ref bool value);

        [DllImport("libeng.dll")]
        public static extern IntPtr engOpen(string startcmd);

        //[DllImport("libeng.dll")]
        //public static extern IntPtr engOpenSingleUse(string startcmd, IntPtr dcom, ref int retstatus);

        [DllImport("libeng.dll")]
        public static extern int engOutputBuffer(IntPtr ep, IntPtr p, int n);

        [DllImport("libeng.dll")]
        public static extern int engPutVariable(IntPtr ep, string name, IntPtr mp);

        [DllImport("libeng.dll")]
        public static extern int engSetVisible(IntPtr ep, [MarshalAs(UnmanagedType.U1)] bool value);
        #endregion

        #region Libmat DLL imports
        //[DllImport("libmat.dll")]
        //public static extern int matClose(IntPtr mfp);

        //[DllImport("libmat.dll")]
        //public static extern int matDeleteVariable(IntPtr mfp, string name);

        //[DllImport("libmat.dll")]
        //public static extern string[] matGetDir(IntPtr mfp, ref int num);

        //[DllImport("libmat.dll")]
        //public static extern IntPtr matGetFp(IntPtr mfp);

        //[DllImport("libmat.dll")]
        //public static extern IntPtr matGetNextVariable(IntPtr mfp, string name);

        //[DllImport("libmat.dll")]
        //public static extern IntPtr matGetNextVariableInfo(IntPtr mfp, string name);

        //[DllImport("libmat.dll")]
        //public static extern IntPtr matGetVariable(IntPtr mfp, string name);

        //[DllImport("libmat.dll")]
        //public static extern IntPtr matGetVariableInfo(IntPtr mfp, string name);

        //[DllImport("libmat.dll")]
        //public static extern IntPtr matOpen(string filename, string mode);

        //[DllImport("libmat.dll")]
        //public static extern int matPutVariable(IntPtr mfp, string name, IntPtr mp);

        //[DllImport("libmat.dll")]
        //public static extern int matPutVariableAsGlobal(IntPtr mfp, string name, IntPtr mp);
        #endregion

        #region Libmx enums
        public enum mxComplexity
        {
            Real,
            Complex
        }

        public enum mxClassID
        {
            Unknown,
            Cell,
            Struct,
            Logical,
            Char,
            Void,
            Double,
            Single,
            Int8,
            Uint8,
            Int16,
            Uint16,
            Int32,
            Uint32,
            Int64,
            Uint64,
            Function,
            Opaque,
            Object
        }
        #endregion

        #region Libmx DLL imports
        //[DllImport("libmx.dll")]
        //public static extern int mxAddField(IntPtr array_ptr, string field_name);

        //[DllImport("libmx.dll")]
        //public static extern string mxArrayToString(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern void mxAssert(int expr, string error_message);

        //[DllImport("libmx.dll")]
        //public static extern void mxAssertS(int expr, string error_message);

        //[DllImport("libmx.dll")]
        //public static extern int mxCalcSingleSubscript(IntPtr array_ptr, int nsubs, int[] subs);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCalloc(uint n, uint size);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateCellArray(int ndim, int[] dims);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateCellMatrix(int m, int n);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateCharArray(int ndim, int[] dims);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateCharMatrixFromStrings(int m, string[][] str);

        [DllImport("libmx.dll")]
        public static extern IntPtr mxCreateDoubleMatrix(int m, int n, mxComplexity ComplexFlag);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateDoubleScalar(double value);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateLogicalArray(int ndim, int[] dims);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateLogicalMatrix(int m, int n);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateLogicalScalar([MarshalAs(UnmanagedType.U1)] bool value);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateNumericArray(int ndim, int[] dims, mxClassID classID, mxComplexity ComplexFlag);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateNumericMatrix(int m, int n, mxClassID classID, mxComplexity ComplexFlag);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateScalarDouble(double value);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateSparse(int m, int n, int nzmax, mxComplexity ComplexFlag);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateSparseLogicalMatrix(int m, int n);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateString(string str);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateStructArray(int ndim, int[] dims, int nfields, string[][] field_names);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxCreateStructMatrix(int m, int n, int nfields, string[][] field_names);

        [DllImport("libmx.dll")]
        public static extern int mxDestroyArray(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxDuplicateArray(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxFree(IntPtr ptr);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetCell(IntPtr array_ptr, int index);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetChars(IntPtr array_ptr);

        [DllImport("libmx.dll")]
        public static extern mxClassID mxGetClassID(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetClassName(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetData(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetDimensions(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern int mxGetElementSize(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern double mxGetEps();

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetField(IntPtr array_ptr, int index, string field_name);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetFieldByNumber(IntPtr array_ptr, int index, int field_number);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetFieldNameByNumber(IntPtr array_ptr, int field_number);

        //[DllImport("libmx.dll")]
        //public static extern int mxGetFieldNumber(IntPtr array_ptr, string field_name);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetImagData(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern double mxGetInf();

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetIr(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetJc(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetLogicals(IntPtr array_ptr);

        [DllImport("libmx.dll")]
        public static extern int mxGetM(IntPtr array_ptr);

        [DllImport("libmx.dll")]
        public static extern int mxGetN(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern double mxGetNaN();

        //[DllImport("libmx.dll")]
        //public static extern int mxGetNumberOfDimensions(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern int mxGetNumberOfElements(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern int mxGetNumberOfFields(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern int mxGetNzmax(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxGetPi(IntPtr array_ptr);

        [DllImport("libmx.dll")]
        public static extern IntPtr mxGetPr(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern double mxGetScalar(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern int mxGetString(IntPtr array_ptr, string buf, int buflen);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsCell(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsChar(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsClass(IntPtr array_ptr, string name);

        [DllImport("libmx.dll")]
        [return: MarshalAs(UnmanagedType.U1)]
        public static extern bool mxIsComplex(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsDouble(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsEmpty(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsFromGlobalWS(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsInf(double value);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsInt8(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsInt16(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsInt32(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsLogical(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsLogicalScalar(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsLogicalScalarTrue(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsNaN(double value);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsNumeric(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsSingle(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsSparse(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //[return: MarshalAs(UnmanagedType.U1)]
        //public static extern bool mxIsStruct(IntPtr array_ptr);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxMalloc(uint n);

        //[DllImport("libmx.dll")]
        //public static extern IntPtr mxRealloc(IntPtr ptr, uint size);

        //[DllImport("libmx.dll")]
        //public static extern void mxRemoveField(IntPtr array_ptr, int field_number);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetAllocFcns(IntPtr callocfcn, IntPtr freefcn, IntPtr reallocfcn, IntPtr mallocfcn);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetCell(IntPtr array_ptr, int index, IntPtr value);

        //[DllImport("libmx.dll")]
        //public static extern int mxSetClassName(IntPtr array_ptr, string classname);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetData(IntPtr array_ptr, IntPtr data_ptr);

        //[DllImport("libmx.dll")]
        //public static extern int mxSetDimensions(IntPtr array_ptr, int[] dims, int ndim);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetField(IntPtr array_ptr, int index, string field_name, IntPtr value);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetFieldByNumber(IntPtr array_ptr, int index, int field_number, IntPtr value);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetImagData(IntPtr array_ptr, IntPtr pi);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetIr(IntPtr array_ptr, int[] ir);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetJc(IntPtr array_ptr, int[] jc);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetM(IntPtr array_ptr, int m);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetN(IntPtr array_ptr, int n);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetNzmax(IntPtr array_ptr, int nzmax);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetPi(IntPtr array_ptr, double[] pi);

        //[DllImport("libmx.dll")]
        //public static extern void mxSetPr(IntPtr array_ptr, double[] pr);
        #endregion
    }
}
